@@ -1,4 +1,4 @@
-__version__ = '1.5.1'
+__version__ = '1.6.0'
 
 import logging
 logging.basicConfig()
@@ -7,7 +7,7 @@ logger = logging.getLogger(__name__)
 from . import syntax
 contract_expression = syntax.contract_expression
 
-from .interface import (Contract, ContractNotRespected,
+from .interface import (Contract, ContractNotRespected, CannotDecorateClassmethods,
                         ContractSyntaxError, ContractException)
 
 from .main import (check, fail, check_multiple, contract_decorator,
@@ -31,11 +31,13 @@ from .enabling import disable_all, enable_all, all_disabled
 from .interface import describe_value, describe_type
 
 # For backwards compatibility
-contracts = contract
+# contracts = contract
 
 # After everything is loaded, load aliases
 from .library import miscellaneous_aliases
 
+from .metaclass import ContractsMeta
+ContractsMeta.__module__ = 'contracts'
 
 # And after everything else is loaded, load the  utils
 from .useful_contracts import *
