@@ -56,7 +56,13 @@ class ContractException(Exception):
 class MissingContract(ContractException):
     pass
 
-class ContractSyntaxError(ContractException):
+class ContractDefinitionError(ContractException):
+    """ Thrown when defining the contracts """
+
+class CannotDecorateClassmethods(ContractDefinitionError):
+    pass
+    
+class ContractSyntaxError(ContractDefinitionError):
     ''' Exception thrown when there is a syntax error in the contracts. '''
 
     def __init__(self, error, where=None):
@@ -99,8 +105,8 @@ class ContractNotRespected(ContractException):
             return contexts
 
         align = []
-        for (contract, context, value) in self.stack:     #@UnusedVariable
-            #cons = ("%s %s" % (contract, contexts)).ljust(30)
+        for (contract, context, value) in self.stack:  # @UnusedVariable
+            # cons = ("%s %s" % (contract, contexts)).ljust(30)
             row = ['checking: %s' % contract,
                     'for value: %s' % describe_value(value, clip=70)]
             align.append(row)
@@ -347,7 +353,7 @@ def describe_value(x, clip=50):
     ''' Describes an object, for use in the error messages. '''
     if hasattr(x, 'shape') and hasattr(x, 'dtype'):
         shape_desc = 'x'.join(str(i) for i in x.shape)
-        desc = 'array[%s](%s) ' % (shape_desc, x.dtype)
+        desc = 'array[%r](%s) ' % (shape_desc, x.dtype)
         final = desc + clipped_repr(x, clip - len(desc))
         return remove_newlines(final)
     else:
